﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml.Controls;
using System.Reflection;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Media;
using Windows.UI;

namespace UWPGridsBenchmark
{
    public abstract class TestGrid : ContentControl
    {
        protected void SetGrid(Control grid)
        {
            this.Content = grid;
        }
        public abstract void Load(IEnumerable data);
        public virtual void Dispose()
        {
            this.Content = null;
        }
        public abstract Task Sort(bool ascending);
        public abstract void ScrollInView(int rowNumber);
        public abstract Task Filter(bool isActive);

        public abstract Task Group();
    }

    public class C1FlexGrid : TestGrid
    {
        C1.Xaml.FlexGrid.C1FlexGrid _grid = null;
        public C1FlexGrid()
        {
            _grid = new C1.Xaml.FlexGrid.C1FlexGrid();
            SetGrid(_grid);
        }

        public override void Dispose()
        {
            var cv = _grid.ItemsSource as C1.Xaml.C1CollectionView;
            if (cv != null)
                cv.Filter = null;
            _grid.ItemsSource = null;
            base.Dispose();
        }

        public override void Load(IEnumerable data)
        {
            _grid.ItemsSource = data;
        }

        public async override Task Sort(bool ascending)
        {
            var cv = _grid.ItemsSource as C1.Xaml.C1CollectionView;
            using (cv.DeferRefresh())
            {
                cv.SortDescriptions.Clear();
                cv.SortDescriptions.Add(new C1.Xaml.SortDescription("ID", ascending ? C1.Xaml.ListSortDirection.Ascending : C1.Xaml.ListSortDirection.Descending));
            }
        }

        public override async Task Group()
        {
            var cv = _grid.ItemsSource as C1.Xaml.C1CollectionView;
            using (cv.DeferRefresh())
            {
                cv.GroupDescriptions.Clear();
                cv.GroupDescriptions.Add(new C1.Xaml.PropertyGroupDescription("CountryID"));
            }
        }

        public async override Task Filter(bool isActive)
        {
            var cv = _grid.ItemsSource as C1.Xaml.C1CollectionView;
            if (cv != null)
            {
                if (isActive)
                    cv.Filter = IsActiveFilter;
                else
                    cv.Filter = IsInactiveFilter;
            }
        }

        bool IsActiveFilter(object customer)
        {
            return ((Customer)customer).Active;
        }

        bool IsInactiveFilter(object customer)
        {
            return !((Customer)customer).Active;
        }

        public override void ScrollInView(int rowNumber)
        {
            _grid.ScrollIntoView(rowNumber, 0);

        }
    }

#if (!RELEASE_TEST)
    public class XuniFlexGrid : TestGrid
    {
        Xuni.UWP.FlexGrid.FlexGrid _grid = null;
        public XuniFlexGrid()
        {
            _grid = new Xuni.UWP.FlexGrid.FlexGrid();
            SetGrid(_grid);
        }

        public override void Dispose()
        {
            _grid.ItemsSource = null;
            base.Dispose();
        }

        public override void Load(IEnumerable data)
        {
            _grid.ItemsSource = data;
        }

        public async override Task Sort(bool ascending)
        {
            var cv = _grid.ItemsSource as Xuni.CollectionView.XuniCollectionView<Customer>;
            if (cv != null)
            {
                var sortDesc = new Xuni.CollectionView.SortDescription("ID", ascending ? Xuni.CollectionView.SortDirection.Ascending : Xuni.CollectionView.SortDirection.Descending);
                await cv.SortAsync(sortDesc);
            }
        }

        public override async Task Group()
        {
            var cv = _grid.ItemsSource as Xuni.CollectionView.XuniCollectionView<Customer>;
            if (cv != null)
            {
                var groupDesc = new Xuni.CollectionView.GroupDescription("CountryID");
                await cv.GroupAsync(groupDesc);
            }
        }

        public async override Task Filter(bool isActive)
        {
            var cv = _grid.ItemsSource as Xuni.CollectionView.XuniCollectionView<Customer>;
            if (cv != null)
            {
                Xuni.CollectionView.FilterUnaryExpression exp = new Xuni.CollectionView.FilterUnaryExpression("Active", Xuni.CollectionView.FilterOperation.Equal, isActive);
                await cv.FilterAsync(exp);
            }
        }
        bool IsActiveFilter(object customer)
        {
            return ((Customer)customer).Active;
        }

        bool IsInactiveFilter(object customer)
        {
            return !((Customer)customer).Active;
        }
        public override void ScrollInView(int rowNumber)
        {
            _grid.ScrollIntoView(rowNumber, 0);
        }
    }
#endif
}
